#include "Camera.h"

void Camera::update() {
    //std::cout << "Position: (" << m_position.x << ", " << m_position.y << ", " << m_position.z << ")" << std::endl;
	glm::mat4 view = glm::mat4(1.0f);
	glm::mat4 proj = glm::mat4(1.0f);

	view = glm::lookAt(m_position, m_position + m_vdirection, m_up);
	proj = glm::perspective(glm::radians(m_fov), m_aspect, m_nearPlane, m_farPlane);

	m_view = view;
	m_proj = proj;
}

void DebugCamera::move(std::vector<Input> inputs) {
	for(const Input& input : inputs)
	{
		if (input == Input::FAST) m_speed = 0.05f;
		if (input == Input::SLOW) m_speed = 0.005f;

		switch (input) {
		case Input::FORWARD:
			m_position += m_speed * m_vdirection;
			break;
		case Input::BACKWARD:
			m_position -= m_speed * m_vdirection;
			break;
		case Input::LEFT:
			m_position -= m_speed * glm::normalize(glm::cross(m_vdirection, m_up));
			break;
		case Input::RIGHT:
			m_position += m_speed * glm::normalize(glm::cross(m_vdirection, m_up));
			break;
		case Input::DOWN:
			m_position += m_speed * m_up;
			break;
		case Input::UP:
			m_position -= m_speed * m_up;
			break;
		default:
			break;
		}
	}
}

void DebugCamera::rotate(int width, int height, int mouse_x, int mouse_y) {
	float rot_x = m_sensitivity * (float)(mouse_y - (height / 2)) / (float)height;
	float rot_y = m_sensitivity * (float)(mouse_x - (width / 2)) / (float)width;

	float pitch = glm::degrees(asin(m_vdirection.y));
	if (pitch + rot_x > 85.0f) {
		rot_x = 85.0f - pitch;
	}
	else if (pitch + rot_x < -85.0f) {
		rot_x = -85.0f - pitch;
	}

	m_vdirection = glm::normalize(glm::rotate(m_vdirection, glm::radians(rot_x), glm::normalize(glm::cross(m_vdirection, m_up))));
	m_vdirection = glm::normalize(glm::rotate(m_vdirection, -glm::radians(rot_y), m_up));
}

void PathCamera::update(float dt) {
	m_position = m_path.getPosition(dt);
	m_vdirection = glm::rotate(m_path.getRotation(dt,glm::vec3(0.f,0.f,-1.f)),glm::vec4(0.f,0.f,-1.f, 1.f));
	this->Camera::update();
	if (m_lookAt != glm::vec3(0.f,0.f,0.f)) {
		m_view = glm::lookAt(m_position, m_lookAt, m_up);
	}
}