#pragma once

#include "Common.h"
#include "glm/gtc/matrix_transform.hpp"
#include "glm/gtx/rotate_vector.hpp"
#include "glm/gtx/vector_angle.hpp"
#include "Path.h"

/**
 *  A base class represents a virtual camera.
 */
class Camera {
public:
	void update();

	glm::mat4 getProjMat() const { return m_proj; };
	glm::mat4 getViewMat() const { return m_view; };
	glm::vec3 getPosition() const { return m_position; }
	void setLookAt(glm::vec3 lookAt) { m_lookAt = lookAt; }

protected:
	Camera() {};
	Camera(uint32_t width, uint32_t height, float fov, float nearPlane, float farPlane) : WIDTH(width), HEIGHT(height), m_fov(fov), m_nearPlane(nearPlane), m_farPlane(farPlane), m_lookAt(glm::vec3(0.f,0.f,0.f)) {};

	glm::mat4 m_proj = glm::mat4(1.0f);
	glm::mat4 m_view = glm::mat4(1.0f);
	bool m_debug = false;

	uint32_t WIDTH;
	uint32_t HEIGHT;

	float m_fov;
	float m_aspect = (float)WIDTH/(float)HEIGHT;
	float m_nearPlane;
	float m_farPlane;

	glm::vec3 m_position = glm::vec3(0.0f, 0.0f, 10.0f);
	glm::vec3 m_vdirection = glm::vec3(0.0f, 0.0f, -1.0f);
	glm::vec3 m_up = glm::vec3(0.0f, 1.0f, 0.0f);
	float m_speed = 0.005f;

	glm::vec3 m_lookAt;
};

/**
 *  A class that implements a debug camera, that is freely movable throughout the scene.
 */
class DebugCamera : public Camera {
public:
	DebugCamera() {};
	DebugCamera(uint32_t width, uint32_t height, float fov, float nearPlane, float farPlane) : Camera(width, height, fov, nearPlane, farPlane) {};

	void move(std::vector<Input> inputs);
	void rotate(int width, int height, int mouse_x, int mouse_y);

private:
	bool m_firstClick = true;
	float m_sensitivity = 160.0f;

};

/**
 *  A class that implements a camera, that follows a given path.
 */
class PathCamera : public Camera {
public:
	PathCamera() {};
	PathCamera(uint32_t width, uint32_t height, float fov, float nearPlane, float farPlane, Path path) : Camera(width, height, fov, nearPlane, farPlane) {	m_path = path; };

	void update(float dt);
private:
	Path m_path;
};