#pragma once

#include "Common.h"
#include "Material.h"
#include "Model.h"
#include "Pipeline.h"
#include "Camera.h"

/**
 *  A class that implements and encapsulates most aspects of a cubemap.
 */
class Cubemap {
public:
	Cubemap() {};
	Cubemap(VulkanDevice vulkanDevice, Pipeline* pipeline, Model* model, CubemapTexture* material, bool dynamic);

	VkBuffer& getVertexBuffer();
	VkBuffer& getIndexBuffer();
	VkDescriptorSet& getDescriptorSet(int frame);
	std::vector<uint32_t> getIndices();
	VkPipeline getPipeline();
	VkPipelineLayout getPipelineLayout();
	CubemapTexture* getTexture();
	VkPipeline getDrawToCubePipeline();
	VkPipelineLayout getDrawToCubePipelineLayout();
	void setDrawToCubePipeline(ImagePipeline* pipeline);
	void setPosition(glm::vec3 position);
	glm::vec3 getPosition();
	VkPipeline getDrawCubeToCubePipeline();
	void setDrawCubeToCubePipeline(Pipeline* pipeline);
	VkPipelineLayout getDrawCubeToCubePipelineLayout();

	void update(uint32_t currentImage, Camera& camera);
	void updateForCubemapDraw(uint32_t currentImage, glm::mat4 viewMat, glm::mat4 projMat);
	void updateWithTRS(uint32_t currentImage, Camera& camera, glm::vec3 translation, float angle, glm::vec3 rotation, glm::vec3 scale);
	void cleanup();

private:
	Model* m_model;
	CubemapTexture* m_material;
	VulkanDevice m_vulkanDevice;
	VkDevice m_device;
	Pipeline* m_pipeline;
	VkPipelineLayout m_pipelineLayout;
	ImagePipeline* m_drawToCubemapPipeline;
	Pipeline* m_drawCubeToCubemapPipeline;
	bool m_dynamic;

	glm::vec3 m_position;
	glm::vec3 m_vdirection;
	glm::mat4 m_model_mat = glm::mat4(1.0f);

	std::vector<VkBuffer> m_uniformBuffers;
	std::vector<VkDeviceMemory> m_uniformBuffersMemory;
	std::vector<void*> m_uniformBuffersMapped;

	std::vector<VkBuffer> m_cameraUniformBuffers;
	std::vector<VkDeviceMemory> m_cameraUniformBuffersMemory;
	std::vector<void*> m_cameraUniformBuffersMapped;

	VkDescriptorPool m_descriptorPool;
	std::vector<VkDescriptorSet> m_descriptorSets;

	void createUniformBuffers();
	void createDescriptorPool();
	void createDescriptorSets();
};