#pragma once

#include "Common.h"

/**
 *  A base class that represents a light source.
 */
class Light {
public:
	float getIntensity() const { return m_intensity; }
	glm::vec3 getColor() const { return m_color; }

protected:
	Light() {};
	Light(float intensity, glm::vec3 color) : m_intensity(intensity), m_color(color) {};

	float m_intensity;
	glm::vec3 m_color;
};

/**
 *  A class that represents a directional light.
 */
class DirectionalLight : public Light {
public:
	DirectionalLight() {};
	DirectionalLight(float intensity, glm::vec3 color, glm::vec3 direction) : Light(intensity, color), m_direction(direction) {};

	glm::vec3 getDirection() const { return m_direction; }

private:
	glm::vec3 m_direction;
};

/**
 *  A class that represents a point light.
 */
class PointLight : public Light {
public:
	PointLight() {};
	PointLight(float intensity, glm::vec3 color, glm::vec3 position) : Light(intensity, color), m_position(position) {};

	glm::vec3 getPosition() const { return m_position; }
private:
	glm::vec3 m_position;
};