#pragma once

#include "Common.h"
#include "VulkanDevice.h"

/**
 *  A class representing the geometry of an object.
 */
class Model {
public:
    Model() {};
    Model(VulkanDevice vulkanDevice, std::string path, bool computeNormals);
    ~Model() {};

    VkBuffer& getVertexBuffer();
    VkBuffer& getIndexBuffer();
    std::vector<uint32_t> getIndices();
    void cleanup();

private:
    VulkanDevice m_vulkanDevice;
    VkDevice m_device;
    VkPhysicalDevice m_physicalDevice;

    std::vector<Vertex> m_vertices;
    std::vector<uint32_t> m_indices;
    VkBuffer m_vertexBuffer;
    VkDeviceMemory m_vertexBufferMemory;
    VkBuffer m_indexBuffer;
    VkDeviceMemory m_indexBufferMemory;

    bool clean = false;

    void loadModel(std::string path);
    void loadWithoutManualCompute(std::string path);
    void createVertexBuffer();
    void createIndexBuffer();
};