#pragma once

#include "Common.h"
#include "Object.h"

/**
 *  A class for storing a set of game objects and handling various operations on them.
 */
class ObjectHandler {
public:
	ObjectHandler() {};
	ObjectHandler(Pipeline* pipeline) : m_pipeline(pipeline->getPipeline()), m_pipelineLayout(pipeline->getPipelineLayout()) {};

	void updateObjects(float t, float dt, uint32_t currentImage, Camera& camera);
	void updateObjectsForCubemapDraw(float t, float dt, uint32_t currentImage, glm::mat4 viewMat, glm::mat4 projMat);
	void addObject(Object* object);
	void removeObject(Object* object);
	void cleanup();
	Object* getObject(int index) { return m_objects[index]; }

	VkPipeline getPipeline() { return m_pipeline; }
	VkPipelineLayout getPipelineLayout() { return m_pipelineLayout; }
	
	std::vector<Object*> getObjects() { return m_objects; }

private:
	std::vector<Object*> m_objects;
	VkPipeline m_pipeline;
	VkPipelineLayout m_pipelineLayout;
};