#include "ParticleSystem.h"

void ParticleSystem::createStorageBuffer() {
    std::default_random_engine rndEngine((unsigned)time(nullptr));
    std::uniform_real_distribution<float> rndDist(0.0f, 1.0f);

    std::vector<glm::vec4> colors;
    colors.push_back(glm::vec4(100.0f/255.0f, 65.0f/255.0f, 23.0f/255.0f, 1.0f));
    colors.push_back(glm::vec4(244.0f / 255.0f, 164.0f / 255.0f, 96.0f / 255.0f, 1.0f));
    colors.push_back(glm::vec4(111.0f / 255.0f, 78.0f / 255.0f, 55.0f / 255.0f, 1.0f));
    colors.push_back(glm::vec4(255.0f / 255.0f, 228.0f / 255.0f, 181.0f / 255.0f, 1.0f));
    colors.push_back(glm::vec4(102.0f / 255.0f, 76.0f / 255.0f, 40.0f / 255.0f, 1.0f));

    std::vector<Particle> particles(m_particleCount);
    int i = 0;
    for (auto& particle : particles) {
        float r = 0.2;
        float theta = rndDist(rndEngine) * 2.0f * M_PI;
        float x = r * cos(theta);
        float y = -3;
        float z = r * sin(theta);
        particle.position = glm::vec3(x, y, z);
        particle.velocity = glm::normalize(glm::vec3(1.0f * rndDist(rndEngine), -1.0f * rndDist(rndEngine), 1.0f * rndDist(rndEngine)));
        
        int randIndex = (int)(((double)std::rand() / (RAND_MAX)) * 4);
        glm::vec4 color;
        switch (randIndex)
        {
        case 0:
            color = colors.at(0);
            break;
        case 1:
            color = colors.at(1);
            break;
        case 2:
            color = colors.at(2);
            break;
        case 3:
            color = colors.at(3);
            break;
        default:
            color = colors.at(4);
            break;
        }
        particle.color = color;
        
        particle.angle = ((double)std::rand() / (RAND_MAX)) * (2.0f * M_PI);
        particle.lifetime = 0.0f;
        particle.maxLifetime = 3.0f + rndDist(rndEngine) * 4.0f;
        particle.randomness = glm::vec3(rndDist(rndEngine) * 2, rndDist(rndEngine), rndDist(rndEngine));
        particle.initial_position = particle.position;
        i++;
    }

    VkDeviceSize bufferSize = sizeof(Particle) * m_particleCount;

    VkBuffer stagingBuffer;
    VkDeviceMemory stagingBufferMemory;
    m_vulkanDevice.createBuffer(bufferSize, VK_BUFFER_USAGE_TRANSFER_SRC_BIT, VK_MEMORY_PROPERTY_HOST_VISIBLE_BIT | VK_MEMORY_PROPERTY_HOST_COHERENT_BIT, stagingBuffer, stagingBufferMemory);

    void* data;
    vkMapMemory(m_device, stagingBufferMemory, 0, bufferSize, 0, &data);
    memcpy(data, particles.data(), (size_t)bufferSize);
    vkUnmapMemory(m_device, stagingBufferMemory);

    m_shaderStorageBuffers.resize(MAX_FRAMES_IN_FLIGHT);
    m_shaderStorageBuffersMemory.resize(MAX_FRAMES_IN_FLIGHT);

    for (size_t i = 0; i < MAX_FRAMES_IN_FLIGHT; i++) {
        m_vulkanDevice.createBuffer(bufferSize, VK_BUFFER_USAGE_STORAGE_BUFFER_BIT | VK_BUFFER_USAGE_VERTEX_BUFFER_BIT | VK_BUFFER_USAGE_TRANSFER_DST_BIT, VK_MEMORY_PROPERTY_DEVICE_LOCAL_BIT, m_shaderStorageBuffers[i], m_shaderStorageBuffersMemory[i]);
        m_vulkanDevice.copyBuffer(stagingBuffer, m_shaderStorageBuffers[i], bufferSize);
    }

    vkDestroyBuffer(m_device, stagingBuffer, nullptr);
    vkFreeMemory(m_device, stagingBufferMemory, nullptr);
};

void ParticleSystem::cleanup() {
    for (int i = 0; i < m_shaderStorageBuffers.size(); i++) {
        vkDestroyBuffer(m_device, m_shaderStorageBuffers[i], nullptr);
        vkFreeMemory(m_device, m_shaderStorageBuffersMemory[i], nullptr);
    }
}