#include "Path.h"
#include <fstream>
#include <sstream>

Path::Path() {
}

Path::~Path() {
}

glm::vec3 Path::getPosition(float dt) {
    return glm::vec3(x(dt),y(dt),z(dt));
}

glm::quat Path::getRotation(float dt, glm::vec3 forward)
{
     glm::vec3 currentPos = getPosition(dt);
     glm::vec3 nextPos = getPosition(dt+0.1f);

     glm::vec3 direction = glm::normalize(nextPos - currentPos);

     glm::vec3 rotationAxis = glm::cross(forward, direction);

     // Handle case when vectors are parallel
     if (glm::length(rotationAxis) < 0.001f) {
         if (glm::dot(forward, direction) > 0)
             return glm::quat(1.0f, 0.0f, 0.0f, 0.0f); // Identity quaternion
         else
             return glm::quat(glm::radians(180.f), 0.0f, 1.0f, 0.0f); // 180-degree rotation
     }

     rotationAxis = glm::normalize(rotationAxis);
     float angle = acos(glm::dot(forward, direction));

     // Create and return quaternion
     return glm::angleAxis(angle, rotationAxis);
}

void Path::loadKeyframes(std::string filepath) {
    std::ifstream file(filepath);
    if (!file.is_open()) {
        std::cerr << "Error opening file: " << filepath << std::endl;
        return;
    }
    std::vector<double> t, x, y, z;
    std::string line;
    while (std::getline(file, line)) {
        std::istringstream ss(line);
        std::string token;

		std::getline(ss, token, ',');
		t.push_back(std::stof(token));
        std::getline(ss, token, ',');
        x.push_back(std::stof(token));
        std::getline(ss, token, ',');
        y.push_back(std::stof(token));
        std::getline(ss, token, ',');
        z.push_back(std::stof(token));
    }
    file.close();
	this->x = tk::spline(t, x);
	this->y = tk::spline(t, y);
	this->z = tk::spline(t, z);

}
