#include "SpaceReEscape.h"

void SpaceReEscape::run() {
	setWorkingDirectory();
    initWindow();
	initVulkan();
    loadAssets();
    renderLoop();
}

void SpaceReEscape::initWindow() {
    glfwInit();
    glfwWindowHint(GLFW_CLIENT_API, GLFW_NO_API);
	glfwWindowHint(GLFW_DECORATED, GLFW_FALSE);

	GLFWmonitor* primaryMonitor = glfwGetPrimaryMonitor();

    m_window = glfwCreateWindow(WIDTH, HEIGHT, "SpaceReEscape", primaryMonitor, nullptr);
    glfwSetWindowUserPointer(m_window, this);
    glfwSetFramebufferSizeCallback(m_window, framebufferResizeCallback);
    m_vulkanHandler = VulkanHandler(m_window);

	glfwSetKeyCallback(m_window, key_callback);
	glfwSetMouseButtonCallback(m_window, mouse_button_callback);

	m_cameraPath = Path();
	m_cameraPath.loadKeyframes("./assets/paths/camera_path.csv");

	m_debugCamera = DebugCamera(WIDTH, HEIGHT, FOV, NEARZ, FARZ);
	m_pathCamera = PathCamera(WIDTH, HEIGHT, FOV, NEARZ, FARZ, m_cameraPath);

	m_directionalLight = DirectionalLight(1.0f, glm::vec3(1.0f), glm::vec3(-1.0f, -1.0f, -1.0f));
}

void SpaceReEscape::initVulkan() {
    m_vulkanHandler.initVulkan();
	m_vulkanDevice = VulkanDevice(m_vulkanHandler.getDevice(), m_vulkanHandler.getPhysicalDevice(), m_vulkanHandler.getCommandPool(),
		m_vulkanHandler.getGraphicsQueue());

	m_imagePipeline = ImagePipeline("./assets/shaders/image_vert.spv", "./assets/shaders/image_frag.spv", m_vulkanDevice, m_vulkanHandler.getDefferedRenderpass(), 5);
	m_cubemapPipeline = CubemapPipeline("./assets/shaders/cubemap_vert.spv", "./assets/shaders/cubemap_frag.spv", m_vulkanDevice, m_vulkanHandler.getDefferedRenderpass(), 5);
	m_particleDrawPipeline = ParticleDrawPipeline("./assets/shaders/particle_vert.spv", "./assets/shaders/particle_frag.spv", "./assets/shaders/particle_geom.spv", m_vulkanDevice, m_vulkanHandler.getDefferedRenderpass(), 5);
	m_particleComputePipeline = ComputePipeline("./assets/shaders/particle_comp.spv", m_vulkanDevice, m_vulkanHandler.getRenderPass());
	m_envMapDrawPipeline = DynEnvironmentMapDrawPipeline("./assets/shaders/envMapDraw_vert.spv", "./assets/shaders/envMapDraw_frag.spv", m_vulkanDevice, m_vulkanHandler.getDefferedRenderpass());
	m_defferedPipeline = RenderDefferedPipeline("./assets/shaders/deffered_vert.spv", "./assets/shaders/deffered_frag.spv", m_vulkanDevice, m_vulkanHandler.getRenderPass());
}

void SpaceReEscape::loadAssets() {
	m_objectHandler = ObjectHandler(&m_imagePipeline);

	Path spaceshipPath = Path();
	spaceshipPath.loadKeyframes("./assets/paths/spaceship_path.csv");

	Model* spaceship = new Model(m_vulkanDevice, spaceship_path, true);
	ImageTexture* spaceship_img = new ImageTexture(m_vulkanDevice, spaceship_texture_path, spaceship_normal_map_path, glm::vec3(0.6f, 0.6f, 1.0f), 1.0f, 0.1f, 0.8f);
    m_objectHandler.addObject(new ImageObject(m_vulkanDevice, &m_imagePipeline, spaceship, spaceship_img, glm::vec3(0.f, -10.f, 40.f), glm::radians(180.f), glm::vec3(1.0f, 0.0f, 0.0f), glm::vec3(0.5f, 0.5f, 0.5f), &spaceshipPath, 1.0f, true));
	//m_objectHandler.addObject(new ImageObject(m_vulkanDevice, &m_imagePipeline, spaceship, spaceship_img, glm::vec3(20.0, 1.0, 5.0), glm::radians(180.f), glm::vec3(1.0f, 0.0f, 0.0f), glm::vec3(0.5f, 0.5f, 0.5f), nullptr, 1.0f));

	Model* ground = new Model(m_vulkanDevice, ground_path, true);
	ImageTexture* ground_img = new ImageTexture(m_vulkanDevice, ground_texture_path, ground_normal_map_path, glm::vec3(0.2f, 1.0f, 0.2f), 0.6f, 0.6f, 0.2f);
	m_objectHandler.addObject(new ImageObject(m_vulkanDevice, &m_imagePipeline, ground, ground_img, glm::vec3(0.0f, 0.0f, 0.0f), glm::radians(180.f), glm::vec3(1.0f, 0.0f, 0.0f), glm::vec3(2.0f, 2.0f, 2.0f), nullptr, 20.0f));

	Model* rock = new Model(m_vulkanDevice, rock_path, true);
	ImageTexture* rock_img = new ImageTexture(m_vulkanDevice, rock_texture_path, rock_normal_map_path, glm::vec3(0.8f, 0.7f, 0.3f), 0.0f, 0.9f, 0.2f);

	Model* redRock = new Model(m_vulkanDevice, red_rock_path, true);
	ImageTexture* redRock_img = new ImageTexture(m_vulkanDevice, red_rock_texture_path, red_rock_normal_map_path, glm::vec3(0.2f, 0.7f, 0.3f), 0.3f, 0.9f, 0.2f);

	Model* redRock2 = new Model(m_vulkanDevice, red_rock2_path, true);
	ImageTexture* redRock_img2 = new ImageTexture(m_vulkanDevice, red_rock2_texture_path, red_rock2_normal_map_path, glm::vec3(0.2f, 0.7f, 0.3f), 0.3f, 0.9f, 0.2f);

	Model* cactus = new Model(m_vulkanDevice, cactus_path, true);
	ImageTexture* cactus_img = new ImageTexture(m_vulkanDevice, cactus_texture_path, cactus_normal_map_path, glm::vec3(0.8f, 0.7f, 0.3f), 0.0f, 0.9f, 0.2f);

	m_objectHandler.addObject(new ImageObject(m_vulkanDevice, &m_imagePipeline, cactus, cactus_img, glm::vec3(20.0f, -3.0f, 0.0f), glm::radians(180.f), glm::vec3(1.0f, 0.0f, 0.0f), glm::vec3(1.0f, 1.0f, 1.0f), nullptr, 1.0f));
	m_objectHandler.addObject(new ImageObject(m_vulkanDevice, &m_imagePipeline, cactus, cactus_img, glm::vec3(-20.0f, -3.0f, 0.0f), glm::radians(180.f), glm::vec3(1.0f, 0.0f, 0.0f), glm::vec3(1.0f, 1.0f, 1.0f), nullptr, 1.0f));

	m_objectHandler.addObject(new ImageObject(m_vulkanDevice, &m_imagePipeline, redRock, redRock_img, glm::vec3(-20.0f, -10.0f, 100.0f), glm::radians(180.f), glm::vec3(1.0f, 0.0f, 0.0f), glm::vec3(1.0f, 1.f, 1.0f), nullptr, 1.0f));

	m_objectHandler.addObject(new ImageObject(m_vulkanDevice, &m_imagePipeline, rock, rock_img, glm::vec3(12.0f, 0.0f, 7.0f), glm::radians(180.f), glm::vec3(1.0f, 0.0f, 0.0f), glm::vec3(7.0f, 7.f, 7.0f), nullptr, 1.0f));
	for (int i = 0; i < 14; i++) {
	 	int rand = (int)(((double)std::rand() / (RAND_MAX)) * 6);

		switch (rand)
		{
		case 0:
			m_objectHandler.addObject(new ImageObject(m_vulkanDevice, &m_imagePipeline, rock, rock_img, glm::vec3(70.0f, 0.0f, 70.0f + i * 10.0f), glm::radians(180.f), glm::vec3(1.0f, 0.0f, 0.0f), glm::vec3(10.0f, 10.f, 10.0f), nullptr, 1.0f));
			m_objectHandler.addObject(new ImageObject(m_vulkanDevice, &m_imagePipeline, redRock, redRock_img, glm::vec3(-200, -30.0f, 150), glm::radians(180.0f), glm::vec3(1.0f, 0.0f, 0.0f), glm::vec3(0.5f, 0.5f, 0.5f), nullptr, 1.0f));
			m_objectHandler.addObject(new ImageObject(m_vulkanDevice, &m_imagePipeline, redRock2, redRock_img2, glm::vec3(550, -10.0f, 350), glm::radians(180.0f), glm::vec3(1.0f, 0.0f, 0.0f), glm::vec3(0.2f, 0.2f, 0.2f), nullptr, 1.0f));
			break;
		case 1:
			m_objectHandler.addObject(new ImageObject(m_vulkanDevice, &m_imagePipeline, rock, rock_img, glm::vec3(70.0f, 0.0f, -70.0f + i * 10.0f), glm::radians(180.f), glm::vec3(1.0f, 1.0f, 0.0f), glm::vec3(10.0f, 10.f + 5.0, 10.0f), nullptr, 1.0f));
			m_objectHandler.addObject(new ImageObject(m_vulkanDevice, &m_imagePipeline, redRock, redRock_img, glm::vec3(300, -30.0f, -100), glm::radians(180.0f), glm::vec3(1.0f, 0.0f, 0.0f), glm::vec3(0.5f, 0.5f, 0.5f), nullptr, 1.0f));
			m_objectHandler.addObject(new ImageObject(m_vulkanDevice, &m_imagePipeline, redRock2, redRock_img2, glm::vec3(400, -20.0f, -200), glm::radians(180.0f), glm::vec3(1.0f, 0.0f, 0.0f), glm::vec3(0.2f, 0.2f, 0.2f), nullptr, 1.0f));
			break;
		case 2:
			m_objectHandler.addObject(new ImageObject(m_vulkanDevice, &m_imagePipeline, rock, rock_img, glm::vec3(70.0f, 0.0f, -70.0f + i * 10.0f), glm::radians(180.f), glm::vec3(1.0f, 0.0f, 1.0f), glm::vec3(10.0f, 10.f + 5.0, 10.0f + 5.0), nullptr, 1.0f));
			m_objectHandler.addObject(new ImageObject(m_vulkanDevice, &m_imagePipeline, redRock, redRock_img, glm::vec3(150.0f, -20.0f, 70.0f), glm::radians(180.0f), glm::vec3(1.0f, 0.0f, 0.0f), glm::vec3(0.5f, 0.5f, 0.5f), nullptr, 1.0f));
			break;
		case 3:
			m_objectHandler.addObject(new ImageObject(m_vulkanDevice, &m_imagePipeline, rock, rock_img, glm::vec3(70.0f, 0.0f, -70.0f + i * 10.0f), glm::radians(180.f), glm::vec3(0.0f, 1.0f, 0.0f), glm::vec3(10.0f + 5.0, 10.f, 10.0f), nullptr, 1.0f));
			break;
		case 4:
			m_objectHandler.addObject(new ImageObject(m_vulkanDevice, &m_imagePipeline, rock, rock_img, glm::vec3(70.0f, 0.0f, -70.0f + i * 10.0f), glm::radians(180.f), glm::vec3(0.0f, 0.0f, 1.0f), glm::vec3(10.0f, 10.f + 5.0, 10.0f), nullptr, 1.0f));
			break;
		default:
			m_objectHandler.addObject(new ImageObject(m_vulkanDevice, &m_imagePipeline, rock, rock_img, glm::vec3(70.0f, 0.0f, -70.0f + i * 10.0f), glm::radians(180.f), glm::vec3(1.0f, 1.0f, 1.0f), glm::vec3(10.0f + 5.0, 10.f, 10.0f), nullptr, 1.0f));
			break;
		}
		
	}

	Model* cube = new Model(m_vulkanDevice, cube_path, false);
	StaticEnvironmentTexture* tenerife = new StaticEnvironmentTexture(m_vulkanDevice, cubemap_texture_basic_path);
	m_cubemap = Cubemap(m_vulkanDevice, &m_cubemapPipeline, cube, tenerife, false /*dynamic*/);

	Model* plane = new Model(m_vulkanDevice, plane_path, true);
	DefferedTexture* defferedTexture = new DefferedTexture(m_vulkanDevice);
	m_defferedPlane = DefferedDrawObject(m_vulkanDevice, &m_defferedPipeline, plane, glm::vec3(0.0f, 0.0f, 0.0f), defferedTexture);
	m_vulkanHandler.createDefferedFramebuffers(&m_defferedPlane);

	ParticleSystem* particleSystem = new ParticleSystem(m_vulkanDevice, 12 * 8192);
	m_particleObject = ParticleObject(m_vulkanDevice, &m_particleComputePipeline, cube, glm::vec3(20.0f, 0.5f, 50.0f), particleSystem, &m_particleDrawPipeline);

	Model* dynCube = new Model(m_vulkanDevice, dynMirror_path, false);
	DynEnvironmentTexture* dynEnvMap = new DynEnvironmentTexture(m_vulkanDevice, 800, 800, 4);
	m_dynEnvironmentMap = Cubemap(m_vulkanDevice, &m_envMapDrawPipeline, dynCube, dynEnvMap, true /*dynamic*/);
	m_vulkanHandler.createDynEnvRenderPass(&m_dynEnvironmentMap);
	m_drawToCubePipeline = ImagePipeline("./assets/shaders/image_to_cube_vert.spv", "./assets/shaders/image_to_cube_frag.spv", m_vulkanDevice, m_vulkanHandler.getDynEnvMapRenderPass(), 1);
	m_cubeToCubemapPipeline = CubemapPipeline("./assets/shaders/cube_to_cube_vert.spv", "./assets/shaders/cube_to_cube_frag.spv", m_vulkanDevice, m_vulkanHandler.getDynEnvMapRenderPass(), 1);
	m_drawParticleToCubePipeline = ParticleDrawPipeline("./assets/shaders/particle_vert.spv", "./assets/shaders/particle_to_cube_frag.spv", "./assets/shaders/particle_geom.spv", m_vulkanDevice, m_vulkanHandler.getDynEnvMapRenderPass(), 1);
	m_dynEnvironmentMap.setDrawToCubePipeline(&m_drawToCubePipeline);
	m_dynEnvironmentMap.setDrawCubeToCubePipeline(&m_cubeToCubemapPipeline);
	m_particleObject.setDrawToCubePipeline(&m_drawParticleToCubePipeline);
	m_dynEnvironmentMap.setPosition(glm::vec3(0.0f, -10.0f, 30.0f));
}

void SpaceReEscape::framebufferResizeCallback(GLFWwindow* window, int width, int height) {
    auto app = reinterpret_cast<SpaceReEscape*>(glfwGetWindowUserPointer(window));
    app->m_framebufferResized = true;
}

void SpaceReEscape::renderLoop() {
	float t = float(glfwGetTime());
	float dt = 0.0f;
	float t_sum = 0.0f;

    while (!glfwWindowShouldClose(m_window)) {

        update(t - t_sum, dt);
        draw(t - t_sum, dt);

		dt = t;
		t = float(glfwGetTime());
		dt = t - dt;

		if ((t - t_sum) > 85.0f) t_sum = t;
    }

    cleanup();
}

void SpaceReEscape::cleanup() {
    m_vulkanHandler.setDeviceWaitIdle();
	m_objectHandler.cleanup();
	m_cubemap.cleanup();
	m_dynEnvironmentMap.cleanup();
	m_particleObject.cleanup();
	m_defferedPlane.cleanup();

	m_vulkanHandler.cleanupDynEnvMap();
	m_vulkanHandler.cleanupDefferedFramebuffers();

	m_imagePipeline.cleanup();
	m_cubemapPipeline.cleanup();
	m_particleComputePipeline.cleanup();
	m_particleDrawPipeline.cleanup();
	m_envMapDrawPipeline.cleanup();
	m_drawToCubePipeline.cleanup();
	m_cubeToCubemapPipeline.cleanup();
	m_drawParticleToCubePipeline.cleanup();
	m_defferedPipeline.cleanup();

    m_vulkanHandler.cleanup();
}

void SpaceReEscape::draw(float t, float dt) {
	DrawBuffer path_drawBuffer;
	if (t <= 30.f) {
		path_drawBuffer = DrawBuffer::REGULAR;
	}
	else if (t <= 35.f) {
		path_drawBuffer = DrawBuffer::ALBEDO;
	}
	else if (t <= 40.f) {
		path_drawBuffer = DrawBuffer::NORMAL;
	}
	else if (t <= 45.f) {
		path_drawBuffer = DrawBuffer::COEFFICIENTS;
	}
	else if (t <= 50.f) {
		path_drawBuffer = DrawBuffer::METALLIC;
	}
	else if (t <= 55.f) {
		path_drawBuffer = DrawBuffer::ROUGHNESS;
	}
	else if (t <= 60.f) {
		path_drawBuffer = DrawBuffer::SPECULARALPHA;
	}
	else if (t <= 65.f) {
		path_drawBuffer = DrawBuffer::WORLDPOS;
	}
	else {
		path_drawBuffer = DrawBuffer::REGULAR;
	}
	if (m_debug) {
		m_vulkanHandler.drawFrame(m_objectHandler, m_debugCamera, t, dt, m_cubemap, m_dynEnvironmentMap, m_particleObject, m_defferedPlane, m_drawBuffer);
	}
	else {
		m_vulkanHandler.drawFrame(m_objectHandler, m_pathCamera, t, dt, m_cubemap, m_dynEnvironmentMap, m_particleObject, m_defferedPlane, path_drawBuffer);
	}
}

void SpaceReEscape::update(float t, float dt) {

	glfwPollEvents();
	glfwGetCursorPos(m_window, &m_mouse_x, &m_mouse_y);

	updateDebugCamera();
	m_pathCamera.update(t);
	

	//TODO: make object update invariant to camera
	if (!m_debug) {
		m_defferedPlane.update(t, dt, m_vulkanHandler.getCurrentFrame(), m_pathCamera, m_directionalLight);
		m_objectHandler.updateObjects(t, dt, m_vulkanHandler.getCurrentFrame(), m_pathCamera);
		m_cubemap.update(m_vulkanHandler.getCurrentFrame(), m_pathCamera);
		m_dynEnvironmentMap.updateWithTRS(m_vulkanHandler.getCurrentFrame(), m_pathCamera, glm::vec3(0.0f, -10.0f, 0.0f), 0, glm::vec3(1.0f, 0.0f, 0.0f), glm::vec3(0.6f, 0.6f, 0.6f));
		m_particleObject.update(t, dt, m_vulkanHandler.getCurrentFrame(), m_pathCamera);

		m_vulkanHandler.dispatchParticleSystemCompute(m_particleObject);
	}
	else { 
		m_defferedPlane.update(t, dt, m_vulkanHandler.getCurrentFrame(), m_debugCamera, m_directionalLight);
		m_dynEnvironmentMap.updateWithTRS(m_vulkanHandler.getCurrentFrame(), m_debugCamera, glm::vec3(0.0f, -10.0f, 0.0f), 0, glm::vec3(1.0f, 0.0f, 0.0f), glm::vec3(0.5f, 0.5f, 0.5f));
		m_cubemap.update(m_vulkanHandler.getCurrentFrame(), m_debugCamera);
		m_particleObject.update(t, dt, m_vulkanHandler.getCurrentFrame(), m_debugCamera);
		m_objectHandler.updateObjects(t, dt, m_vulkanHandler.getCurrentFrame(), m_debugCamera);

		m_vulkanHandler.dispatchParticleSystemCompute(m_particleObject);
	} 
}

void SpaceReEscape::setWorkingDirectory() {
	std::filesystem::path workingDir = std::filesystem::current_path();

	while (!workingDir.empty()) {
		if (std::filesystem::exists(workingDir / "assets")) {
			std::filesystem::current_path(workingDir);
			return;
		}
		workingDir = workingDir.parent_path();
	}

	throw std::runtime_error("Project root not found. Assets folder missing.");
}

void SpaceReEscape::updateDebugCamera() {
	std::vector<Input> inputs;
	int width, height;
	glfwGetWindowSize(m_window, &width, &height);

	if (m_fast) {
		inputs.push_back(Input::FAST);
	}
	else {
		inputs.push_back(Input::SLOW);
	}

	if (m_backward) {
		inputs.push_back(Input::BACKWARD);
	}
	if (m_forward) {
		inputs.push_back(Input::FORWARD);
	}
	if (m_right) {
		inputs.push_back(Input::RIGHT);
	}
	if (m_left) {
		inputs.push_back(Input::LEFT);
	}
	if (m_upward) {
		inputs.push_back(Input::UP);
	}
	if (m_downward) {
		inputs.push_back(Input::DOWN);
	}

	m_debugCamera.move(inputs);

	if (m_debug_mouse_control) {
		if (m_firstClick) {
			glfwSetCursorPos(m_window, (width / 2), (height / 2));
			m_firstClick = false;
			m_mouse_x = width / 2;
			m_mouse_y = height / 2;
		}
		
		m_debugCamera.rotate(width, height, (int)m_mouse_x, (int)m_mouse_y);

		glfwSetCursorPos(m_window, (width / 2), (height / 2));
	}
	else {
		m_firstClick = true;
	}	
	
	m_debugCamera.update();
}

void SpaceReEscape::key_callback(GLFWwindow* window, int key, int scancode, int action, int mods)
{
	// F1 - Debug Camera
	// Esc - Exit
	// W,A,S,D,Q,E - Debug Movement
	// LeftShift - Toggle Fast Debug Camera

	auto app = reinterpret_cast<SpaceReEscape*>(glfwGetWindowUserPointer(window));

	if (app->m_debug) {
		if (key == GLFW_KEY_W && action == GLFW_PRESS) {
			app->m_forward = true;
		}
		if (key == GLFW_KEY_S && action == GLFW_PRESS) {
			app->m_backward = true;
		}
		if (key == GLFW_KEY_D && action == GLFW_PRESS) {
			app->m_right = true;
		}
		if (key == GLFW_KEY_A && action == GLFW_PRESS) {
			app->m_left = true;
		}
		if (key == GLFW_KEY_Q && action == GLFW_PRESS) {
			app->m_downward = true;
		}
		if ((key == GLFW_KEY_E && action == GLFW_PRESS) || (key == GLFW_KEY_SPACE && action == GLFW_PRESS)) {
			app->m_upward = true;
		}
		if (key == GLFW_KEY_LEFT_SHIFT && action == GLFW_PRESS) {
			app->m_fast = true;
		}
		if (key == GLFW_KEY_W && action == GLFW_RELEASE) {
			app->m_forward = false;
		}
		if (key == GLFW_KEY_S && action == GLFW_RELEASE) {
			app->m_backward = false;
		}
		if (key == GLFW_KEY_D && action == GLFW_RELEASE) {
			app->m_right = false;
		}
		if (key == GLFW_KEY_A && action == GLFW_RELEASE) {
			app->m_left = false;
		}
		if (key == GLFW_KEY_Q && action == GLFW_RELEASE) {
			app->m_downward = false;
		}
		if ((key == GLFW_KEY_E && action == GLFW_RELEASE) || (key == GLFW_KEY_SPACE && action == GLFW_RELEASE)) {
			app->m_upward = false;
		}
		if (key == GLFW_KEY_LEFT_SHIFT && action == GLFW_RELEASE) {
			app->m_fast = false;
		}
	}

	if (action != GLFW_RELEASE) return;

	switch (key)
	{
	case GLFW_KEY_ESCAPE:
		glfwSetWindowShouldClose(window, true);
		break;
	case GLFW_KEY_F1:
		app->m_debug = !app->m_debug;
		break;
	case GLFW_KEY_F3:
		app->m_drawBuffer = DrawBuffer::REGULAR;
		break;
	case GLFW_KEY_F4:
		app->m_drawBuffer = DrawBuffer::ALBEDO;
		break;
	case GLFW_KEY_F5:
		app->m_drawBuffer = DrawBuffer::NORMAL;
		break;
	case GLFW_KEY_F6:
		app->m_drawBuffer = DrawBuffer::COEFFICIENTS;
		break;
	case GLFW_KEY_F7:
		app->m_drawBuffer = DrawBuffer::METALLIC;
		break;
	case GLFW_KEY_F8:
		app->m_drawBuffer = DrawBuffer::ROUGHNESS;
		break;
	case GLFW_KEY_F9:
		app->m_drawBuffer = DrawBuffer::SPECULARALPHA;
		break;
	case GLFW_KEY_F10:
		app->m_drawBuffer = DrawBuffer::WORLDPOS;
		break;
	default:
		break;
	}
};

void SpaceReEscape::mouse_button_callback(GLFWwindow* window, int button, int action, int mods)
{
	auto app = reinterpret_cast<SpaceReEscape*>(glfwGetWindowUserPointer(window));

	if (button == GLFW_MOUSE_BUTTON_LEFT && action == GLFW_PRESS) {

		if (!app->m_debug_mouse_control) {
			glfwSetInputMode(window, GLFW_CURSOR, GLFW_CURSOR_HIDDEN);
			app->m_debug_mouse_control = true;
		}
		else {
			glfwSetInputMode(window, GLFW_CURSOR, GLFW_CURSOR_NORMAL);
			app->m_debug_mouse_control = false;
		}
	}
}
   



