#pragma once

#define GLFW_INCLUDE_VULKAN
#include <glfw3.h>

#include "Common.h"
#include "Camera.h"
#include "VulkanHandler.h"
#include "Path.h"

/**
 *  A class handling the general execution of the program.
 */
class SpaceReEscape {
public:
    void run();

private:
    const float ASPECT = float(WIDTH) / float(HEIGHT);
    const float FOV = 60.0f;

    GLFWwindow* m_window;
    bool m_framebufferResized = false;

    VulkanHandler m_vulkanHandler; // = VulkanHandler(window);
    VulkanDevice m_vulkanDevice;
    ObjectHandler m_objectHandler;

    Cubemap m_cubemap;
    Cubemap m_dynEnvironmentMap;
    ParticleObject m_particleObject;
    DefferedDrawObject m_defferedPlane;

    ImagePipeline m_imagePipeline;
    CubemapPipeline m_cubemapPipeline;
    ParticleDrawPipeline m_particleDrawPipeline;
    ComputePipeline m_particleComputePipeline;
    DynEnvironmentMapDrawPipeline m_envMapDrawPipeline;
    ImagePipeline m_drawToCubePipeline;
    CubemapPipeline m_cubeToCubemapPipeline;
    ParticleDrawPipeline m_drawParticleToCubePipeline;
    RenderDefferedPipeline m_defferedPipeline;

    const std::string spaceship_path = "./assets/models/spaceship.obj";
    const std::string spaceship_texture_path = "./assets/textures/spaceship.jpg";
	const std::string spaceship_normal_map_path = "./assets/textures/spaceship_normal.jpg";

    const std::string cube_path = "./assets/models/cube.obj";
    const std::string cubemap_texture_basic_path = "./assets/textures/sea";

    const std::string ground_path = "./assets/models/ground.obj";
    const std::string ground_texture_path = "./assets/textures/ground.jpg";
	const std::string ground_normal_map_path = "./assets/textures/ground_normal.jpg";

    const std::string rock_path = "./assets/models/rock.obj";
    const std::string rock_texture_path = "./assets/textures/rock.jpg";
	const std::string rock_normal_map_path = "./assets/textures/rock_normal.jpg";

    const std::string red_rock_path = "./assets/models/redRock.obj";
    const std::string red_rock_texture_path = "./assets/textures/redRock.jpg";
    const std::string red_rock_normal_map_path = "./assets/textures/redRock_normal.jpg";

    const std::string red_rock2_path = "./assets/models/redRock2.obj";
    const std::string red_rock2_texture_path = "./assets/textures/redRock2.jpg";
    const std::string red_rock2_normal_map_path = "./assets/textures/redRock2_normal.jpg";

    const std::string plane_path = "./assets/models/plane.obj";

	const std::string dynMirror_path = "./assets/models/dynCube.obj";

	const std::string cactus_path = "./assets/models/cactus.obj";
	const std::string cactus_texture_path = "./assets/textures/cactus.png";
	const std::string cactus_normal_map_path = "./assets/textures/cactus_normal.jpg";

    DebugCamera m_debugCamera;
    PathCamera m_pathCamera;

    DirectionalLight m_directionalLight;

    bool m_debug = false;
    bool m_forward = false;
    bool m_backward = false;
    bool m_right = false;
    bool m_left = false;
    bool m_upward = false;
    bool m_downward = false;
    bool m_fast = false;
    bool m_firstClick = true;
    bool m_debug_mouse_control = false;
    double m_mouse_x;
    double m_mouse_y; 

    DrawBuffer m_drawBuffer = DrawBuffer::REGULAR;

	Path m_cameraPath;

    void initWindow();

    void initVulkan();

    static void framebufferResizeCallback(GLFWwindow* window, int width, int height);

    void renderLoop();

    void loadModel();

    void draw(float t, float dt);

    void update(float t, float dt);

    void cleanup();

    void loadAssets();

    static void key_callback(GLFWwindow* window, int key, int scancode, int action, int mods);

    static void mouse_button_callback(GLFWwindow* window, int button, int action, int mods);

    void updateDebugCamera();

    void setWorkingDirectory();
};