#version 450

layout(binding = 1) uniform samplerCube cubemapSampler;

layout(binding = 2) uniform CameraUniformBuffer {
    vec4 position;
} cubo;

layout(binding = 3) uniform samplerCube depthSampler;

layout(location = 0) in vec3 fragUVW;
layout(location = 1) in vec3 fragNormal;
layout(location = 5) in vec3 inWorldPos;
layout(location = 8) in vec3 reflectionDir;
layout(location = 9) in vec3 boxMax;
layout(location = 10) in vec3 boxMin;

layout(location = 0) out vec4 outColor;
layout(location = 1) out vec4 outNormal;
layout(location = 2) out vec4 outCoefficients;
layout(location = 3) out vec4 outMaterial;
layout(location = 4) out vec4 outWorldPos;

void main() {
	vec3 cameraWorldPos = cubo.position.xyz;
	vec3 normal = normalize(fragNormal);
		
	vec3 viewDir = inWorldPos - cameraWorldPos;
	vec3 reflectionDir = normalize(reflect(viewDir, normal));
	
	// parallax-correction
	vec3 FirstPlaneIntersect = (boxMax - inWorldPos) / reflectionDir;
	vec3 SecondPlaneIntersect = (boxMin - inWorldPos) / reflectionDir;
	vec3 FurthestPlane = max(FirstPlaneIntersect, SecondPlaneIntersect);
	float Distance = min(min(FurthestPlane.x, FurthestPlane.y), FurthestPlane.z);

	vec3 IntersectPositionWS = inWorldPos + reflectionDir * Distance;
	reflectionDir = IntersectPositionWS - vec3(0.0f, -10.0f, 30.0f);
	//reflectionDir = IntersectPositionWS - inWorldPos;
	
	vec4 reflectionColor = texture(cubemapSampler, reflectionDir);
	
    outColor = mix(reflectionColor, vec4(0.68f, 0.68f, 0.68f, 1.0f), 0.2);
	outNormal = vec4(normalize(fragNormal), 1.0f);
	outCoefficients = vec4(0.6f, 0.6f, 1.0f,1.0f);
	outMaterial = vec4(0.0f, 0.1f, 0.8f,1.0f);
	outWorldPos = vec4(inWorldPos, 1.0f);
}