#version 450

layout(points) in;
layout(triangle_strip, max_vertices = 14) out;

layout(location = 0) in vec4[] inColor;
layout(location = 1) in float[] inLifetime;
layout(location = 2) in float[] inMaxLifetime;

layout(binding = 0) uniform UniformBufferObject {
	mat4 model;
    mat4 view;
    mat4 proj;
} ubo;

layout(push_constant) uniform PerViewPush {
	mat4 vdirection;
} push;

layout(location = 0) out vec4 outColor;
layout(location = 1) out vec3 outNormal;
layout(location = 5) out vec3 outWorldPos;

void main() {
	
	vec4 center = gl_in[0].gl_Position;
	
	float remaining = (inMaxLifetime[0] - inLifetime[0]) / inMaxLifetime[0];
	
	float size = 0.1f * (inLifetime[0] / inMaxLifetime[0]);

    vec4 vertices[8] = vec4[](
        vec4(size, size, size, 0.0), 	
		vec4(-size, size, size, 0.0), 
		vec4(size, size, -size, 0.0), 
		vec4(-size, size, -size, 0.0), 	
		
		vec4(size, -size, size, 0.0), 
		vec4(-size, -size, size, 0.0), 	
		vec4(-size, -size, -size, 0.0), 
		vec4(size, -size, -size, 0.0)
    );

    int indices[14] = int[](
        3, 2, 6, 7, 4, 2, 0,
		3, 1, 6, 5, 4, 1, 0
    );

    for (int i = 0; i < 14; i += 3) {
        vec4 v0 = center + vertices[indices[i]];
        vec4 v1 = center + vertices[indices[i + 1]];
        vec4 v2 = center + vertices[indices[i + 2]];

        vec3 edge1 = (v1 - v0).xyz;
        vec3 edge2 = (v2 - v0).xyz;
        vec3 normal = normalize(cross(edge1, edge2));

        for (int j = 0; j < 3; j++) {
            vec4 vertexPos = center + vertices[indices[i + j]];
			
            vec4 worldPos = ubo.model * vertexPos;
            outWorldPos = worldPos.xyz;
		
            gl_Position = ubo.proj * ubo.view * push.vdirection * ubo.model * vertexPos;
            outColor = vec4(inColor[0].rgb, 1 - inLifetime[0] / inMaxLifetime[0] * 0.4f);
            outNormal = normal;
            EmitVertex();
        }
    }
	
	
    EndPrimitive();
}